# @push.rocks/tapbundle

A test automation library bundling utilities and tools for TAP (Test Anything Protocol) based testing, specifically tailored for tapbuffer.

## Features

- TAP (Test Anything Protocol) compliant output
- Support for both browser and Node.js environments
- Built-in test coordination and timing measurements
- Parallel and sequential test execution
- Pre-task support for setup operations
- Rich assertion library via `expect`
- Comprehensive Node.js tooling (shell, crypto, MongoDB, S3)

## Installation

```bash
npm install @push.rocks/tapbundle --save-dev
```

## Usage

The `@push.rocks/tapbundle` is a versatile testing framework compatible with TAP, designed using TypeScript to facilitate robust and scalable testing of applications, whether you are dealing with unit tests, integration tests, or simply need a streamlined way to automate assertions across your application's lifecycle. The framework is especially useful if you are already using or planning to use tapbuffer.

### Core API

The main test runner is accessed through the `tap` object:

```typescript
tap.test(
  description: string,           // Test description
  testFunction: async (tools?: TapTools) => Promise<any>,  // Test implementation
  modeArg: 'normal' | 'only' | 'skip' = 'normal'          // Execution mode
): TapTest
```

### Getting Started

1. **Basic Setup**

   First, import the necessary modules:

   ```typescript
   import { tap, expect, expectAsync } from '@push.rocks/tapbundle';
   ```

   Start with a simple test:

   ```typescript
   tap.test('Initial test succeeds', async () => {
     return expect(true).toBeTrue();
   });

   tap.start();
   ```

2. **Defining and Organizing Tests**

   ```typescript
   tap.test('basic arithmetic test', async () => {
     expect(1 + 1).toEqual(2);
   });

   // Async Tests
   tap.test('async operation test', async () => {
     const fetchData = async () => Promise.resolve('data');
     await expectAsync(fetchData()).toBeResolved();
   });
   ```

3. **Tools for Advanced Testing**

   ```typescript
   // Delay and Timing
   tap.test('test with delay', async (tools) => {
     await tools.delayFor(500); // waits for 500ms
     expect(true).toBeTrue();
   });

   // Custom Pre Tasks
   tap.preTask('initialize environment', async () => {
     console.log('Setting up preconditions');
   });
   ```

4. **Execution Control**

   ```typescript
   // Skip tests
   tap.skip.test('skip this test', async () => {
     // This test will be ignored
   });

   // Exclusive tests
   tap.only.test('run this test exclusively', async () => {
     // Only this test will run
   });

   // Colored output
   tap.test('test with colored output', async (tools) => {
     const message = await tools.coloredString('Test Passed!', 'green');
     console.log(message);
   });
   ```

5. **Node.js Integration**

   ```typescript
   import { tapNodeTools } from '@push.rocks/tapbundle/node';

   tap.test('execute shell command', async () => {
     const result = await tapNodeTools.runCommand('ls -la');
     expect(result.exitCode).toEqual(0);
   });

   tap.test('create HTTPS certificate', async () => {
     const { key, cert } = await tapNodeTools.createHttpsCert('localhost');
     expect(key).toInclude('-----BEGIN RSA PRIVATE KEY-----');
   });
   ```

6. **Environment Variables**

   ```typescript
   tap.test('use environment variable', async (tools) => {
     const dbUrl = await tools.getEnvVarOnDemand('DB_URL');
     expect(dbUrl).toBeDefined();
   });
   ```

7. **Asynchronous Control**

   ```typescript
   tap.test('controlled async scenario', async (tools) => {
     const asyncOp = async () => Promise.resolve('complete');
     tools.timeout(1000); // if operation exceeds 1000ms, test fails
     const result = await asyncOp();
     expect(result).toBe('complete');
   });
   ```

8. **Web Testing**

   ```typescript
   import { webhelpers } from '@push.rocks/tapbundle';

   tap.test('web component test', async () => {
     const element = await webhelpers.fixture(
       webhelpers.html`<my-element></my-element>`
     );
     expect(element.shadowRoot.querySelector('div')).toBeDefined();
   });
   ```

9. **Database and Storage Testing**

   ```typescript
   tap.test('Smartmongo setup test', async () => {
     const smartmongo = await tapNodeTools.createSmartmongo();
     await smartmongo.stop();
   });

   tap.test('Smarts3 setup', async () => {
     const smarts3 = await tapNodeTools.createSmarts3();
     console.log('Smarts3 running');
     await smarts3.stop();
   });
   ```

## Best Practices

1. Use descriptive test names that explain the expected behavior
2. Leverage pre-tasks for common setup operations
3. Use `testParallel` for independent tests that can run concurrently
4. Utilize test coordination for dependent tests
5. Set appropriate timeouts for async operations

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
